package com.altinity.opsview.odw.reports.test;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfree.chart.JFreeChart;

import com.altinity.opsview.odw.dao.ODWDAO;
import com.altinity.opsview.odw.model.ServiceCheck;
import com.altinity.opsview.odw.model.ServiceCheckGroup;
import com.altinity.opsview.odw.model.WeeklyServiceAvailability;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.jfreechart.BarChart;
import com.excira.jfreechart.BarData;


public class TestSummaryChart extends TestBase {
	
	/**
	 * 
	 */
	public void testCreateChart() {
		
		// Setup report criteria
		ReportCriteria rc = new ReportCriteria();
		rc.setReportId(2);
		
		try {
			
			SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
			Date dataStartDate = formatter.parse("2007-11-05");
			rc.setReportStartDate(dataStartDate);
			
			Calendar cal = Calendar.getInstance();
			cal.setTime(dataStartDate);
			cal.add(Calendar.WEEK_OF_YEAR, 1);
			rc.setReportEndDate(cal.getTime());
		}
		catch(ParseException e) {
			System.out.println("Error attempting to parse date");
			e.printStackTrace();
		}		
		
		// Get the keywords and corresponding servicechecks for this report
		List <String> keywords = reportsDAO.listKeywordsForReportId(rc);
		List <ServiceCheck> serviceChecks = odwDAO.listServiceChecksByKeywords(rc, keywords);
		
		// Generate map of servicecheck hostname/name key to servicecheck ids
		Map <String,ServiceCheckGroup> keyedSCs = new HashMap<String, ServiceCheckGroup>();
		for ( ServiceCheck sc : serviceChecks ) {
			
			// Get the hostname/name key for this servicecheck
			String key = sc.getHostName() + sc.getName();
			
			// Get the corresponding list for this key
			ServiceCheckGroup group = keyedSCs.get(key);
			
			// Create the list if it doesn't
			// already exist
			if ( group == null ) {
				
				group = new ServiceCheckGroup();
				keyedSCs.put(key, group);
			}
			
			// Add servicecheck to group
			group.addServiceCheck(sc);
		}
		
		// Now query availability for each servicecheck and generate the corresponding
		// bar data
		List <BarData> data = new ArrayList<BarData>();
		int index = 0;
		for ( String key : keyedSCs.keySet() ) {
			
			ServiceCheckGroup group = keyedSCs.get(key);
			
			// Add the availability for this servicecheck to the list
			WeeklyServiceAvailability summary = odwDAO.getWeeklyAvailabilityForServiceCheck(rc, group.getIds());
			
			String text = group.getText();
			
			BarData barData = new BarData(summary.getPercentOk(), ODWDAO.PERCENT_OK_INDEX, text + index);
			data.add(barData);
			
			barData = new BarData(summary.getPercentScheduled(), ODWDAO.PERCENT_SCHED_INDEX, text + index);
			data.add(barData);
			
			barData = new BarData(summary.getPercentUnscheduled(), ODWDAO.PERCENT_UNSCHED_INDEX, text + index);
			data.add(barData);
			
			index++;
		}
		
		BarChart barChart = new BarChart();
		JFreeChart chart = barChart.createChart("", "", data, false);
		saveImageToTempDir(chart, "testSummary");
	}
}
