package com.excira.jfreechart;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.awt.Color;
import java.awt.Paint;
import java.util.Iterator;
import java.util.List;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.LegendItem;
import org.jfree.chart.LegendItemCollection;
import org.jfree.chart.LegendItemSource;
import org.jfree.chart.axis.Axis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTickUnit;
import org.jfree.chart.axis.SubCategoryAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.renderer.AbstractRenderer;
import org.jfree.chart.renderer.category.StackedBarRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.Dataset;

public class BarChart implements Chart {
	
	private SubCategoryAxis domainAxis;
	private NumberAxis rangeAxis;
	private boolean hideLegend = false;
	
	public JFreeChart createChart(String chartTitle, String seriesName, List data, boolean allowUpdates) {

		// Generate visual representation of single (x,y)
		StackedBarRenderer renderer = (StackedBarRenderer) getRenderer();
		
		// Set up the domain (x) axis
		SubCategoryAxis categoryAxis = (SubCategoryAxis) getDomainAxis();

		// Set up the range (y) axis
		ValueAxis valueAxis = (ValueAxis) getRangeAxis(data, seriesName);

		// Get the data series from the specified bar data
		DefaultCategoryDataset dataset = (DefaultCategoryDataset) getDataSet("", data, false);
		CategoryPlot plot = new CategoryPlot(dataset, categoryAxis, valueAxis, renderer);
		plot.setBackgroundPaint(Color.WHITE);
		plot.setFixedLegendItems(getLegend());
		plot.setRenderer(renderer);
		
		
		// Finally! Create that chart!
		JFreeChart chart = new JFreeChart(chartTitle, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
		chart.setBackgroundPaint(Color.WHITE);
		return chart;
	}

	/**
	 * 
	 */
	public Axis getDomainAxis() {

		if ( domainAxis == null ) {
			domainAxis = new SubCategoryAxis("Date");
			domainAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_90);
		}
		return domainAxis;
	}
	
	/**
	 * 
	 */
	public void createDomainAxis(String text, CategoryLabelPositions labelPosition) {
		
		domainAxis = new SubCategoryAxis(text);
		domainAxis.setCategoryLabelPositions(labelPosition);
	}
	
	/**
	 * 
	 */
	public void createRangeAxis(List data, String label, double tickUnit) {
		
		rangeAxis = new NumberAxis(label);
		rangeAxis.setRange(0.0d, 100.0d);
		rangeAxis.setTickUnit(new NumberTickUnit(tickUnit));
	}

	/**
	 * 
	 */
	public Axis getRangeAxis(List data, String label) {

		if ( rangeAxis == null ) {
			rangeAxis = new NumberAxis(label);
			rangeAxis.setRange(0.0d, 100.0d);
			rangeAxis.setTickUnit(new NumberTickUnit(5.0d));
		}
		return rangeAxis;
	}

	/**
	 * 
	 */
	public Dataset getDataSet(String seriesName, List data, boolean allowUpdates) {

		DefaultCategoryDataset dataSet = new DefaultCategoryDataset();

		Iterator i = data.iterator();
		while (i.hasNext()) {

			BarData barData = (BarData) i.next();
			dataSet.addValue(barData.getValue(), barData.getRowKey(), barData.getColumnKey());
		}

		return dataSet;
	}

	/**
	 * 
	 */
	public LegendItemSource getRenderer() {

		StackedBarRenderer renderer = new StackedBarRenderer();
		
		// Create first series paint
		renderer.setSeriesPaint(0, BarChart.getGreenPaint());

		// Create second series paint
		renderer.setSeriesPaint(1, BarChart.getYellowPaint());

		// Create third series paint
		renderer.setSeriesPaint(2, BarChart.getRedPaint());

		return renderer;
	}
	
	/**
	 * 
	 */
	public static Paint getGreenPaint() {

		return new Color(0, 102, 51);
	}

	/**
	 * 
	 */
	public static Paint getYellowPaint() {

		return new Color(225, 225, 0);
	}

	/**
	 * 
	 */
	public static Paint getRedPaint() {

		return new Color(225, 0, 0);
	}
	
	/**
	 * 
	 */
	public void setHideLegend(boolean hideLegend) {
		
		this.hideLegend = hideLegend;
	}

	/**
	 * 
	 */
	private LegendItemCollection getLegend() {

		LegendItemCollection legend = new LegendItemCollection();
		if ( hideLegend )
			return legend;

		LegendItem percentOK = new LegendItem("Availability", "", "", "", AbstractRenderer.DEFAULT_SHAPE,
				AbstractRenderer.DEFAULT_STROKE, BarChart.getGreenPaint());

		LegendItem percentSched = new LegendItem("Scheduled Downtime", "", "", "", AbstractRenderer.DEFAULT_SHAPE,
				AbstractRenderer.DEFAULT_STROKE, BarChart.getYellowPaint());

		LegendItem percentUnsched = new LegendItem("Unscheduled Downtime", "", "", "", AbstractRenderer.DEFAULT_SHAPE,
				AbstractRenderer.DEFAULT_STROKE, BarChart.getRedPaint());

		legend.add(percentOK);
		legend.add(percentSched);
		legend.add(percentUnsched);

		return legend;
	}
}
